@@ -16,18 +16,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* $Id: bzero.c,v 1.1.1.1 1999/10/15 22:49:23 nmav Exp $ */
+
 #ifndef LIBDEFS_H
 # define LIBDEFS_H
 # include <libdefs.h>
 #endif
 
-void Bzero(void *s, int n)
+void Bzero(void *s, size_t n)
 {
 #ifdef HAVE_MEMSET
-	memset(s, '\0', n);
+	memset( (void*) s, '\0', (size_t) n);
 #else
 # ifdef HAVE_BZERO
-	bzero(s, n);
+	bzero( (void*) s, (size_t) n);
 # else
 	char *stmp = s;
 
