@@ -12,6 +12,8 @@
 /*                                                                      */
 /* Dr Brian Gladman (gladman@seven77.demon.co.uk) 14th January 1999     */
 
+/* $Id: cast-256.c,v 1.2 1999/10/17 09:28:44 nmav Exp $ */
+
 /* modified for mcrypt */
 
 /* Timing data for CAST-256 (cast.c)
@@ -406,11 +408,6 @@ void _mcrypt_cast256_encrypt(word32 * l_key, word32 * blk)
 	blk[1] = byteswap(blk[1]);
 	blk[2] = byteswap(blk[2]);
 	blk[3] = byteswap(blk[3]);
-/*
-#else
-    blk[0] = blk[0]; blk[1] = blk[1];
-    blk[2] = blk[2]; blk[3] = blk[3];
-*/
 #endif
 
 	f_rnd(blk, 0);
@@ -431,11 +428,6 @@ void _mcrypt_cast256_encrypt(word32 * l_key, word32 * blk)
 	blk[1] = byteswap(blk[1]);
 	blk[2] = byteswap(blk[2]);
 	blk[3] = byteswap(blk[3]);
-/*
-#else
-    blk[0] = (blk[0]); blk[1] = (blk[1]);
-    blk[2] = (blk[2]); blk[3] = (blk[3]);
-*/
 #endif
 }
 
@@ -450,11 +442,6 @@ void _mcrypt_cast256_decrypt(word32 * l_key, word32 * blk)
 	blk[1] = byteswap(blk[1]);
 	blk[2] = byteswap(blk[2]);
 	blk[3] = byteswap(blk[3]);
-/*
-#else
-    blk[0] = (blk[0]); blk[1] = (blk[1]);
-    blk[2] = (blk[2]); blk[3] = (blk[3]);
-*/
 #endif
 	f_rnd(blk, 88);
 	f_rnd(blk, 80);
@@ -474,11 +461,6 @@ void _mcrypt_cast256_decrypt(word32 * l_key, word32 * blk)
 	blk[1] = byteswap(blk[1]);
 	blk[2] = byteswap(blk[2]);
 	blk[3] = byteswap(blk[3]);
-/*
-#else
-    blk[0] = (blk[0]); blk[1] = (blk[1]);
-    blk[2] = (blk[2]); blk[3] = (blk[3]);
-*/
 #endif
 
 }
