@@ -17,6 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* $Id: mcrypt.c,v 1.8 1999/10/20 09:13:41 nmav Exp $ */
+
 #ifndef LIBDEFS_H
 #define LIBDEFS_H
 #include <libdefs.h>
@@ -40,6 +42,9 @@
 #include <saferplus.h>
 #include <loki97.h>
 #include <rijndael.h>
+#include <serpent.h>
+#include <arcfour.h>
+#include <enigma.h>
 
 /* I think now is re-entrant */
 
@@ -59,6 +64,8 @@ pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
 static int cur_thread = 0;	/* No threads */
 #endif
 
+
+
 /* Up to three keys for an algorithm */
 static word8 **akey = NULL;
 static word8 **bkey = NULL;
@@ -126,15 +133,14 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 
 	switch (algorithm) {
 
-	case DES:
+	case MCRYPT_DES:
 		_mcrypt_desinit(0);
 		akey[thread] = mxcalloc(1, 8 * 16 * sizeof(char));
 		_mcrypt_des_setkey((void *) akey[thread],
 				   (void *) keyword_given[thread]);
 		return thread;
-		break;
 
-	case SAFER64:
+	case MCRYPT_SAFER_64:
 		akey[thread] =
 		    mxcalloc(1, 1 +
 			     SAFER_BLOCK_LEN * (1 +
@@ -149,55 +155,59 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 					     1, akey[thread]);
 		/* 8 rounds and secure key schedule */
 		return thread;
-		break;
 
-	case CAST128:
+	case MCRYPT_CAST_128:
 		akey[thread] = mxcalloc(1, sizeof(struct cast_key));
 		_mcrypt_cast_setkey((void *) akey[thread],
 				    (void *) keyword_given[thread],
 				    lenofkey);
 		return thread;
-		break;
 
-	case LOKI97:
+	case MCRYPT_LOKI97:
 		akey[thread] = mxcalloc(1, 96 * sizeof(word32));
 		_mcrypt_loki97_set_key((void *) akey[thread],
 				       (void *) keyword_given[thread],
 				       lenofkey);
 		return thread;
-		break;
 
-	case CAST256:
+	case MCRYPT_CAST_256:
 		akey[thread] = mxcalloc(1, 96 * sizeof(word32));
 		_mcrypt_cast256_set_key((void *) akey[thread],
 					(void *) keyword_given[thread],
 					lenofkey);
 		return thread;
-		break;
 
 
-	case RIJNDAEL128:
-	case RIJNDAEL192:
-	case RIJNDAEL256:
+	case MCRYPT_RIJNDAEL_128:
+	case MCRYPT_RIJNDAEL_192:
+	case MCRYPT_RIJNDAEL_256:
 		akey[thread] = mxcalloc(1, sizeof(RI));
 		_mcrypt_rijndael_gentables();
-		_mcrypt_rijndael_set_key((void *) akey[thread], 4, lenofkey/4, (void *) keyword_given[thread]);
+		_mcrypt_rijndael_set_key((void *) akey[thread], 4,
+					 lenofkey / 4,
+					 (void *) keyword_given[thread]);
 		return thread;
-		break;
 
-	case RC2_128:
-	case RC2_1024:
-	case RC2_256:
+	case MCRYPT_SERPENT_128:
+	case MCRYPT_SERPENT_192:
+	case MCRYPT_SERPENT_256:
+		akey[thread] = mxcalloc(1, sizeof(SERPENT_KEY));
+		_mcrypt_serpent_set_key((void *) akey[thread],
+				       (void *) keyword_given[thread], lenofkey);
+		return thread;
+
+	case MCRYPT_RC2_128:
+	case MCRYPT_RC2_1024:
+	case MCRYPT_RC2_256:
 		akey[thread] = mxcalloc(1, 64 * sizeof(word16));
 		_mcrypt_rc2_keyschedule((void *) akey[thread],
 					(void *) keyword_given[thread],
 					lenofkey);
 		return thread;
-		break;
 
 #ifdef NON_FREE
 
-	case IDEA:
+	case MCRYPT_IDEA:
 		akey[thread] = mxcalloc(1, 52 * sizeof(word16));
 		bkey[thread] = mxcalloc(1, 52 * sizeof(word16));
 		_mcrypt_Idea_ExpandUserKey((void *) keyword_given[thread],
@@ -205,21 +215,31 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 		_mcrypt_Idea_InvertKey((void *) akey[thread],
 				       (void *) bkey[thread]);
 		return thread;
-		break;
 
-	case RC6_128:
-	case RC6_192:
-	case RC6_256:
+	case MCRYPT_RC6_128:
+	case MCRYPT_RC6_192:
+	case MCRYPT_RC6_256:
 		akey[thread] = mxcalloc(1, 44 * sizeof(word32));
 		_mcrypt_rc6_set_key((void *) akey[thread],
 				    (void *) keyword_given[thread],
 				    lenofkey);
 		return thread;
-		break;
+
+	case MCRYPT_ARCFOUR:
+		akey[thread] = mxcalloc(1, sizeof(arcfour_key));
+		_mcrypt_arcfour_prepare_key( (void *) keyword_given[thread],
+				    lenofkey, (void *) akey[thread]);
+		return thread;
 
 #endif
+	case MCRYPT_CRYPT:
+		akey[thread] = mxcalloc(1, sizeof(CRYPT_KEY));
+		_mcrypt_set_key_crypt( (void *) akey[thread], (void *) keyword_given[thread],
+				    lenofkey);
+		return thread;
 
-	case SAFER128:
+
+	case MCRYPT_SAFER_128:
 		_mcrypt_Safer_Init_Module();
 		akey[thread] =
 		    mxcalloc(1, 1 +
@@ -236,9 +256,8 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 		/* 8 rounds and secure key schedule */
 
 		return thread;
-		break;
 
-	case TripleDES:
+	case MCRYPT_3DES:
 		_mcrypt_desinit(0);
 
 		akey[thread] = mxcalloc(1, 8 * 16 * sizeof(char));
@@ -255,20 +274,18 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 				   (void *) &(keyword_given[thread])[16]);
 
 		return thread;
-		break;
 
-	case SAFERPLUS:
+	case MCRYPT_SAFERPLUS:
 		akey[thread] = mxcalloc(1, sizeof(SPI));
 		_mcrypt_saferplus_set_key((void *) akey[thread],
 					  (void *) keyword_given[thread],
 					  lenofkey);
 		return thread;
-		break;
 
-	case BLOWFISH448:
-	case BLOWFISH128:
-	case BLOWFISH192:
-	case BLOWFISH256:
+	case MCRYPT_BLOWFISH_448:
+	case MCRYPT_BLOWFISH_128:
+	case MCRYPT_BLOWFISH_192:
+	case MCRYPT_BLOWFISH_256:
 		akey[thread] = mxcalloc(1, sizeof(blf_ctx));
 		if (lenofkey < 2) {
 			_mcrypt_blf_key((void *) akey[thread],
@@ -279,27 +296,23 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key,
 					lenofkey);
 		}
 		return thread;
-		break;
 
-	case TWOFISH128:
-	case TWOFISH192:
-	case TWOFISH256:
+	case MCRYPT_TWOFISH_128:
+	case MCRYPT_TWOFISH_192:
+	case MCRYPT_TWOFISH_256:
 		akey[thread] = mxcalloc(1, sizeof(TWI));
 		_mcrypt_twofish_set_key((void *) akey[cur_thread],
 					(void *) keyword_given[thread],
 					lenofkey);
 		return thread;
-		break;
 
-	case GOST:
+	case MCRYPT_GOST:
 		_mcrypt_kboxinit();
 		return thread;
-		break;
 
-	case ThreeWAY:
-	case TEAN:
+	case MCRYPT_3WAY:
+	case MCRYPT_XTEA:
 		return thread;
-		break;
 
 	default:
 		return -1;
@@ -318,112 +331,104 @@ int mcrypt(int thread, void *plaintext)
 
 	switch (algorithm_given[thread]) {
 
-	case SAFER64:
-	case SAFER128:
+	case MCRYPT_SAFER_64:
+	case MCRYPT_SAFER_128:
 		_mcrypt_Safer_Encrypt_Block(plaintext, akey[thread]);
 		return 0;
-		break;
 
-	case SAFERPLUS:
+	case MCRYPT_SAFERPLUS:
 		_mcrypt_saferplus_encrypt((void *) akey[thread],
 					  plaintext);
 		return 0;
-		break;
 
-	case CAST128:
+	case MCRYPT_CAST_128:
 		_mcrypt_cast_encrypt((void *) akey[thread],
 				     (void *) plaintext);
 		return 0;
-		break;
 
-	case LOKI97:
+	case MCRYPT_LOKI97:
 		_mcrypt_loki97_encrypt((void *) akey[thread],
 				       (void *) plaintext);
 		return 0;
-		break;
 
-	case CAST256:
+	case MCRYPT_CAST_256:
 		_mcrypt_cast256_encrypt((void *) akey[thread],
 					(void *) plaintext);
 		return 0;
-		break;
 
-	case RC2_128:
-	case RC2_256:
-	case RC2_1024:
+	case MCRYPT_RC2_128:
+	case MCRYPT_RC2_256:
+	case MCRYPT_RC2_1024:
 		_mcrypt_rc2_encrypt((void *) akey[thread],
 				    (void *) plaintext);
 		return 0;
-		break;
 
 #ifdef NON_FREE
-	case IDEA:
+	case MCRYPT_IDEA:
 		_mcrypt_Idea_Crypt((void *) plaintext, (void *) plaintext,
 				   (void *) akey[thread]);
 		return 0;
-		break;
 
-	case RC6_128:
-	case RC6_192:
-	case RC6_256:
+	case MCRYPT_RC6_128:
+	case MCRYPT_RC6_192:
+	case MCRYPT_RC6_256:
 		_mcrypt_rc6_encrypt((void *) akey[thread],
 				    (void *) plaintext);
 		return 0;
-		break;
+
 
 #endif
 
-	case RIJNDAEL128:
-	case RIJNDAEL192:
-	case RIJNDAEL256:
+	case MCRYPT_RIJNDAEL_128:
+	case MCRYPT_RIJNDAEL_192:
+	case MCRYPT_RIJNDAEL_256:
 		_mcrypt_rijndael_encrypt((void *) akey[thread],
 					 (void *) plaintext);
 		return 0;
-		break;
 
-	case DES:
+	case MCRYPT_SERPENT_128:
+	case MCRYPT_SERPENT_192:
+	case MCRYPT_SERPENT_256:
+		_mcrypt_serpent_encrypt((void *) akey[thread],
+					(void *) plaintext);
+		return 0;
+
+	case MCRYPT_DES:
 		_mcrypt_endes((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case TripleDES:
+	case MCRYPT_3DES:
 		_mcrypt_endes((void *) akey[thread], plaintext);
 		_mcrypt_dedes((void *) bkey[thread], plaintext);
 		_mcrypt_endes((void *) ckey[thread], plaintext);
 		return 0;
-		break;
 
-	case BLOWFISH448:
-	case BLOWFISH128:
-	case BLOWFISH192:
-	case BLOWFISH256:
+	case MCRYPT_BLOWFISH_448:
+	case MCRYPT_BLOWFISH_128:
+	case MCRYPT_BLOWFISH_192:
+	case MCRYPT_BLOWFISH_256:
 		_mcrypt_enblf((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case TWOFISH128:
-	case TWOFISH192:
-	case TWOFISH256:
+	case MCRYPT_TWOFISH_128:
+	case MCRYPT_TWOFISH_192:
+	case MCRYPT_TWOFISH_256:
 		_mcrypt_twofish_encrypt((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case GOST:
+	case MCRYPT_GOST:
 		_mcrypt_gostcrypt(plaintext,
 				  (void *) keyword_given[thread]);
 		return 0;
-		break;
 
-	case TEAN:
+	case MCRYPT_XTEA:
 		_mcrypt_cl_enc_block((void *) keyword_given[thread],
 				     plaintext);
 		return 0;
-		break;
 
-	case ThreeWAY:
+	case MCRYPT_3WAY:
 		_mcrypt_en3way(plaintext, (void *) keyword_given[thread]);
 		return 0;
-		break;
 
 	default:
 		return 0;
@@ -437,113 +442,104 @@ int mdecrypt(int thread, void *plaintext)
 {
 	switch (algorithm_given[thread]) {
 
-	case SAFER64:
-	case SAFER128:
+	case MCRYPT_SAFER_64:
+	case MCRYPT_SAFER_128:
 		_mcrypt_Safer_Decrypt_Block(plaintext,
 					    (void *) akey[thread]);
 		return 0;
-		break;
 
-	case SAFERPLUS:
+	case MCRYPT_SAFERPLUS:
 		_mcrypt_saferplus_decrypt((void *) akey[thread],
 					  plaintext);
 		return 0;
-		break;
 
-	case CAST128:
+	case MCRYPT_CAST_128:
 		_mcrypt_cast_decrypt((void *) akey[thread],
 				     (void *) plaintext);
 		return 0;
-		break;
 
-	case LOKI97:
+	case MCRYPT_LOKI97:
 		_mcrypt_loki97_decrypt((void *) akey[thread],
 				       (void *) plaintext);
 		return 0;
-		break;
 
-	case CAST256:
+	case MCRYPT_CAST_256:
 		_mcrypt_cast256_decrypt((void *) akey[thread],
 					(void *) plaintext);
 		return 0;
-		break;
 
-	case RC2_128:
-	case RC2_256:
-	case RC2_1024:
+	case MCRYPT_RC2_128:
+	case MCRYPT_RC2_256:
+	case MCRYPT_RC2_1024:
 		_mcrypt_rc2_decrypt((void *) akey[thread],
 				    (void *) plaintext);
 		return 0;
-		break;
 
 #ifdef NON_FREE
-	case IDEA:
+	case MCRYPT_IDEA:
 		_mcrypt_Idea_Crypt((void *) plaintext, (void *) plaintext,
 				   (void *) bkey[thread]);
 		return 0;
-		break;
 
-	case RC6_128:
-	case RC6_192:
-	case RC6_256:
+	case MCRYPT_RC6_128:
+	case MCRYPT_RC6_192:
+	case MCRYPT_RC6_256:
 		_mcrypt_rc6_decrypt((void *) akey[thread],
 				    (void *) plaintext);
 		return 0;
-		break;
 
 #endif
 
-	case RIJNDAEL128:
-	case RIJNDAEL192:
-	case RIJNDAEL256:
+	case MCRYPT_RIJNDAEL_128:
+	case MCRYPT_RIJNDAEL_192:
+	case MCRYPT_RIJNDAEL_256:
 		_mcrypt_rijndael_decrypt((void *) akey[thread],
 					 (void *) plaintext);
 		return 0;
-		break;
 
-	case DES:
+	case MCRYPT_SERPENT_128:
+	case MCRYPT_SERPENT_192:
+	case MCRYPT_SERPENT_256:
+		_mcrypt_serpent_decrypt((void *) akey[thread],
+					(void *) plaintext);
+		return 0;
+
+	case MCRYPT_DES:
 		_mcrypt_dedes((char (*)[8]) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case TEAN:
+	case MCRYPT_XTEA:
 		_mcrypt_cl_dec_block((void *) keyword_given[thread],
 				     plaintext);
 		return 0;
-		break;
 
-	case TripleDES:
+	case MCRYPT_3DES:
 		_mcrypt_dedes((void *) ckey[thread], plaintext);
 		_mcrypt_endes((void *) bkey[thread], plaintext);
 		_mcrypt_dedes((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case TWOFISH128:
-	case TWOFISH192:
-	case TWOFISH256:
+	case MCRYPT_TWOFISH_128:
+	case MCRYPT_TWOFISH_192:
+	case MCRYPT_TWOFISH_256:
 		_mcrypt_twofish_decrypt((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case BLOWFISH448:
-	case BLOWFISH128:
-	case BLOWFISH192:
-	case BLOWFISH256:
+	case MCRYPT_BLOWFISH_448:
+	case MCRYPT_BLOWFISH_128:
+	case MCRYPT_BLOWFISH_192:
+	case MCRYPT_BLOWFISH_256:
 		_mcrypt_deblf((void *) akey[thread], plaintext);
 		return 0;
-		break;
 
-	case GOST:
+	case MCRYPT_GOST:
 		_mcrypt_gostdecrypt(plaintext,
 				    (void *) keyword_given[thread]);
 		return 0;
-		break;
 
-	case ThreeWAY:
+	case MCRYPT_3WAY:
 		_mcrypt_de3way(plaintext, (void *) keyword_given[thread]);
 		return 0;
-		break;
 
 	default:
 		return 0;
@@ -560,88 +556,93 @@ int end_mcrypt(int td)
 
 	switch (algorithm_given[td]) {
 
-	case RIJNDAEL128:
-	case RIJNDAEL192:
-	case RIJNDAEL256:
+	case MCRYPT_RIJNDAEL_128:
+	case MCRYPT_RIJNDAEL_192:
+	case MCRYPT_RIJNDAEL_256:
 		mxfree(akey[td], sizeof(RI));
 		break;
 
+	case MCRYPT_SERPENT_128:
+	case MCRYPT_SERPENT_192:
+	case MCRYPT_SERPENT_256:
+		mxfree(akey[td], sizeof(SERPENT_KEY));
+		break;
 
-	case DES:
+
+	case MCRYPT_DES:
 		_mcrypt_desdone();
 		mxfree(akey[td], 8 * 16 * sizeof(char));
 		break;
 
-	case TripleDES:
+	case MCRYPT_3DES:
 		_mcrypt_desdone();
 		mxfree(akey[td], 8 * 16 * sizeof(char));
 		mxfree(bkey[td], 8 * 16 * sizeof(char));
 		mxfree(ckey[td], 8 * 16 * sizeof(char));
 		break;
 
-	case RC2_128:
-	case RC2_256:
-	case RC2_1024:
+	case MCRYPT_RC2_128:
+	case MCRYPT_RC2_256:
+	case MCRYPT_RC2_1024:
 		mxfree(akey[td], 64 * sizeof(word16));
 		break;
 
 #ifdef NON_FREE
-	case IDEA:
+	case MCRYPT_IDEA:
 		mxfree(akey[td], 52 * sizeof(word16));
 		mxfree(bkey[td], 52 * sizeof(word16));
 		break;
 
-	case RC6_128:
-	case RC6_192:
-	case RC6_256:
+	case MCRYPT_RC6_128:
+	case MCRYPT_RC6_192:
+	case MCRYPT_RC6_256:
 		mxfree(akey[td], 44 * sizeof(word32));
 		break;
 
-#endif
-
-	case SERPENT128:
-	case SERPENT192:
-	case SERPENT256:
-		mxfree(akey[td], 140 * sizeof(word32));
+	case MCRYPT_ARCFOUR:
+		mxfree(akey[td], sizeof(arcfour_key));
 		break;
-
-	case TWOFISH128:
-	case TWOFISH192:
-	case TWOFISH256:
+#endif
+	case MCRYPT_CRYPT:
+		mxfree(akey[td], sizeof(CRYPT_KEY));
+		break;
+		
+	case MCRYPT_TWOFISH_128:
+	case MCRYPT_TWOFISH_192:
+	case MCRYPT_TWOFISH_256:
 		mxfree(akey[td], sizeof(TWI));
 		break;
 
-	case BLOWFISH448:
-	case BLOWFISH128:
-	case BLOWFISH192:
-	case BLOWFISH256:
+	case MCRYPT_BLOWFISH_448:
+	case MCRYPT_BLOWFISH_128:
+	case MCRYPT_BLOWFISH_192:
+	case MCRYPT_BLOWFISH_256:
 		mxfree(akey[td], sizeof(blf_ctx));
 		break;
 
-	case TEAN:
-	case GOST:
-	case ThreeWAY:
+	case MCRYPT_XTEA:
+	case MCRYPT_GOST:
+	case MCRYPT_3WAY:
 		break;
 
-	case CAST128:
+	case MCRYPT_CAST_128:
 		mxfree(akey[td], sizeof(struct cast_key));
 		break;
 
-	case LOKI97:
+	case MCRYPT_LOKI97:
 		mxfree(akey[td], 96 * sizeof(word32));
 		break;
 
-	case CAST256:
+	case MCRYPT_CAST_256:
 		mxfree(akey[td], 96 * sizeof(word32));
 		break;
 
-	case SAFER64:
-	case SAFER128:
+	case MCRYPT_SAFER_64:
+	case MCRYPT_SAFER_128:
 		mxfree(akey[td],
 		       1 + SAFER_BLOCK_LEN * (1 +
 					      2 * SAFER_MAX_NOF_ROUNDS));
 		break;
-		break;
 
 	default:
 		return 0;
@@ -658,7 +659,7 @@ int init_mcrypt_ecb(const int algorithm, void *key, int lenofkey)
 
 	LOCK;
 
-	if (cur_thread == MAX_THREADS) {
+	if (cur_thread == MAX_THREADS-1) {
 		cur_thread = 0;
 		used_all_threads = 1;
 	}
@@ -670,7 +671,7 @@ int init_mcrypt_ecb(const int algorithm, void *key, int lenofkey)
 		UNLOCK;
 		return -1;
 	}
-	x = init_mcrypt(ECB, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_ECB, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt(cur_thread);
@@ -718,6 +719,80 @@ int mdecrypt_ecb(int td, void *ciphertext, int len)
 
 
 
+/* STREAM MODE */
+int init_mcrypt_stream (const int algorithm, void *key, int lenofkey)
+{
+	int x;
+
+	LOCK;
+
+	if (cur_thread == MAX_THREADS-1) {
+		cur_thread = 0;
+		used_all_threads = 1;
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+	}
+
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	}
+	x = init_mcrypt(MCRYPT_STREAM, algorithm, cur_thread, key, lenofkey);
+
+	if (x < 0) {
+		end_mcrypt(cur_thread);
+		UNLOCK;
+		return -1;
+	} else {
+		UNLOCK;
+		return x;
+	}
+
+}
+
+
+
+
+int mcrypt_stream(int td, void *plaintext, int len)
+{
+	switch (algorithm_given[td]) {
+
+#ifdef NON_FREE	
+	case MCRYPT_ARCFOUR:
+		_mcrypt_arcfour( (void *) plaintext, len, (void *) akey[td]);
+		return 0;
+#endif
+	case MCRYPT_CRYPT:
+		_mcrypt_encrypt_crypt( (void*) akey[td], (void *) plaintext, len);
+		return 0;
+	default:
+		return -1;
+	}
+
+}
+
+
+
+int mdecrypt_stream(int td, void *ciphertext, int len)
+{
+	switch (algorithm_given[td]) {
+
+#ifdef NON_FREE	
+	case MCRYPT_ARCFOUR:
+		_mcrypt_arcfour( (void *) ciphertext, len, (void *) akey[td]);
+		return 0;
+#endif
+	case MCRYPT_CRYPT:
+		_mcrypt_decrypt_crypt( (void*) akey[td], (void *) ciphertext, len);
+		return 0;
+	default:
+		return -1;
+	}
+}
+
+
+
 
 /* CBC MODE */
 
@@ -744,7 +819,7 @@ int init_mcrypt_cbc(const int algorithm, void *key, int lenofkey)
 
 	LOCK;
 
-	if (cur_thread == MAX_THREADS) {
+	if (cur_thread == MAX_THREADS-1) {
 		cur_thread = 0;
 		used_all_threads = 1;
 	}
@@ -780,7 +855,7 @@ int init_mcrypt_cbc(const int algorithm, void *key, int lenofkey)
 	    mxcalloc(1, mcrypt_get_block_size(algorithm));
 /* End cbc */
 
-	x = init_mcrypt(CBC, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_CBC, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_cbc(cur_thread);
@@ -843,7 +918,7 @@ int init_mcrypt_cbc_iv(const int algorithm, void *key, int lenofkey,
 		mcrypt_get_block_size(algorithm));
 /* End cbc */
 
-	x = init_mcrypt(CBC, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_CBC, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_cbc(cur_thread);
@@ -934,7 +1009,7 @@ int init_mcrypt_cfb(const int algorithm, void *key, int lenofkey, void *IV)
 
 	LOCK;
 
-	if (cur_thread == MAX_THREADS) {
+	if (cur_thread == MAX_THREADS-1) {
 		cur_thread = 0;
 		used_all_threads = 1;
 	}
@@ -979,7 +1054,7 @@ int init_mcrypt_cfb(const int algorithm, void *key, int lenofkey, void *IV)
 
 /* End cfb */
 
-	x = init_mcrypt(CFB, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_CFB, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_cfb(cur_thread);
@@ -1067,7 +1142,7 @@ int init_mcrypt_ofb(const int algorithm, void *key, int lenofkey, void *IV)
 
 	LOCK;
 
-	if (cur_thread == MAX_THREADS) {
+	if (cur_thread == MAX_THREADS-1) {
 		cur_thread = 0;
 		used_all_threads = 1;
 	}
@@ -1112,7 +1187,7 @@ int init_mcrypt_ofb(const int algorithm, void *key, int lenofkey, void *IV)
 
 /* End ofb */
 
-	x = init_mcrypt(OFB, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_OFB, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_ofb(cur_thread);
@@ -1202,7 +1277,7 @@ int init_mcrypt_nofb(const int algorithm, void *key, int lenofkey,
 
 	LOCK;
 
-	if (cur_thread == MAX_THREADS) {
+	if (cur_thread == MAX_THREADS-1) {
 		cur_thread = 0;
 		used_all_threads = 1;
 	}
@@ -1247,7 +1322,7 @@ int init_mcrypt_nofb(const int algorithm, void *key, int lenofkey,
 
 /* End nofb */
 
-	x = init_mcrypt(nOFB, algorithm, cur_thread, key, lenofkey);
+	x = init_mcrypt(MCRYPT_nOFB, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_nofb(cur_thread);
@@ -1313,53 +1388,59 @@ int mdecrypt_nofb(int td, void *plaintext, int len)
 int mcrypt_generic_init(const int mode, const int algorithm, void *key,
 			int lenofkey, void *IV)
 {
+	if (is_block_algorithm(algorithm) != 0) {
+		switch (mode) {
+		case MCRYPT_CBC:
+			if (IV == NULL) {
+				return init_mcrypt_cbc(algorithm, key,
+						       lenofkey);
+			} else {
+				return init_mcrypt_cbc_iv(algorithm, key,
+							  lenofkey, IV);
+			}
 
-	switch (mode) {
-	case CBC:
-		if (IV == NULL) {
-			return init_mcrypt_cbc(algorithm, key, lenofkey);
-		} else {
-			return init_mcrypt_cbc_iv(algorithm, key, lenofkey,
-						  IV);
+		case MCRYPT_ECB:
+			return init_mcrypt_ecb(algorithm, key, lenofkey);
+		case MCRYPT_CFB:
+			return init_mcrypt_cfb(algorithm, key, lenofkey,
+					       IV);
+		case MCRYPT_OFB:
+			return init_mcrypt_ofb(algorithm, key, lenofkey,
+					       IV);
+		case MCRYPT_nOFB:
+			return init_mcrypt_nofb(algorithm, key, lenofkey,
+						IV);
+		default:
+			return -1;
 		}
-		break;
-	case ECB:
-		return init_mcrypt_ecb(algorithm, key, lenofkey);
-
-		break;
-	case CFB:
-		return init_mcrypt_cfb(algorithm, key, lenofkey, IV);
-		break;
-	case OFB:
-		return init_mcrypt_ofb(algorithm, key, lenofkey, IV);
-		break;
-	case nOFB:
-		return init_mcrypt_nofb(algorithm, key, lenofkey, IV);
-		break;
-	default:
-		return -1;
+	} else {
+		if (mode==MCRYPT_STREAM) {
+	        	return init_mcrypt_stream(algorithm, key, lenofkey);
+	        } else {
+	        	return -1;
+	        }
 	}
+
+
 }
 
 int mcrypt_generic(int td, void *plaintext, int len)
 {
 
 	switch (mode_given[td]) {
-	case CBC:
+	case MCRYPT_CBC:
 		return mcrypt_cbc(td, plaintext, len);
-		break;
-	case ECB:
+	case MCRYPT_ECB:
 		return mcrypt_ecb(td, plaintext, len);
-		break;
-	case CFB:
+	case MCRYPT_CFB:
 		return mcrypt_cfb(td, plaintext, len);
-		break;
-	case OFB:
+	case MCRYPT_OFB:
 		return mcrypt_ofb(td, plaintext, len);
-		break;
-	case nOFB:
+	case MCRYPT_nOFB:
 		return mcrypt_nofb(td, plaintext, len);
-		break;
+	case MCRYPT_STREAM:
+		return mcrypt_stream(td, plaintext, len);
+		
 	default:
 		return -1;
 	}
@@ -1370,21 +1451,19 @@ int mdecrypt_generic(int td, void *ciphertext, int len)
 {
 
 	switch (mode_given[td]) {
-	case CBC:
+	case MCRYPT_CBC:
 		return mdecrypt_cbc(td, ciphertext, len);
-		break;
-	case ECB:
+	case MCRYPT_ECB:
 		return mdecrypt_ecb(td, ciphertext, len);
-		break;
-	case CFB:
+	case MCRYPT_CFB:
 		return mdecrypt_cfb(td, ciphertext, len);
-		break;
-	case OFB:
+	case MCRYPT_OFB:
 		return mdecrypt_ofb(td, ciphertext, len);
-		break;
-	case nOFB:
+	case MCRYPT_nOFB:
 		return mdecrypt_nofb(td, ciphertext, len);
-		break;
+	case MCRYPT_STREAM:
+		return mdecrypt_stream(td, ciphertext, len);
+
 	default:
 		return -1;
 	}
@@ -1395,21 +1474,19 @@ int mcrypt_generic_end(const int td)
 {
 
 	switch (mode_given[td]) {
-	case CBC:
+	case MCRYPT_CBC:
 		return end_mcrypt_cbc(td);
-		break;
-	case ECB:
+	case MCRYPT_ECB:
 		return end_mcrypt_ecb(td);
-		break;
-	case CFB:
+	case MCRYPT_CFB:
 		return end_mcrypt_cfb(td);
-		break;
-	case OFB:
+	case MCRYPT_OFB:
 		return end_mcrypt_ofb(td);
-		break;
-	case nOFB:
+	case MCRYPT_nOFB:
 		return end_mcrypt_nofb(td);
-		break;
+	case MCRYPT_STREAM:
+		return end_mcrypt_stream(td);
+		
 	default:
 		return -1;
 	}
