@@ -1,8 +1,3 @@
-#ifndef LIBDEFS_H
-# define LIBDEFS_H
-# include <libdefs.h>
-#endif
-
 typedef struct {
     word32 state[5];
     word32 count[2];
