@@ -3,12 +3,14 @@
 #include <strings.h>
 #include <stdlib.h>
 
+/* $Id: mode_test.c,v 1.2 1999/10/16 15:18:14 nmav Exp $ */
+
 /* Prints plaintext and ciphertext in hex for all the algorithms */
 
 int main()
 {
 
-	int td, i;
+	int td, i, j, td2;
 	int x = 0, mode;
 	unsigned char *keyword = NULL;
 	unsigned char *plaintext;
@@ -27,6 +29,85 @@ int main()
 			if (is_ok_mode(mode) != 0)
 				continue;
 
+			if (is_block_algorithm(i) == 0) {
+				if (mode!=0) continue;
+
+				keyword = malloc(24);
+				strcpy(keyword, "Ena poly megalo keyword");
+
+				bzero(cipher_tmp, sizeof(cipher_tmp));
+				td =
+				    mcrypt_generic_init(MCRYPT_STREAM, i,
+							keyword, 24, NULL);
+
+				td2 =
+				    mcrypt_generic_init(MCRYPT_STREAM, i,
+							keyword, 24, NULL);
+
+				if (td < 0)
+					continue;
+
+				name = mcrypt_get_algorithms_name(i);
+				if (name == NULL)
+					continue;
+				printf("Algorithm: %s... ", name);
+
+				printf("Mode: STREAM... ");
+
+
+				ciphertext = malloc(10);
+				plaintext = malloc(10);
+				bzero(plaintext, 10);
+				bzero(ciphertext, 10);
+
+#ifdef DEBUG
+				printf("\nplaintext:  ");
+				for (j = 0; j < 10; j++) {
+					printf("%.2x", ciphertext[j]);
+				}
+#endif
+
+#ifdef DEBUG
+				printf("\nkeyword:    ");
+				for (j = 0; j < 24; j++) {
+					printf("%.2x", keyword[j]);
+				}
+#endif
+
+				mcrypt_generic(td, ciphertext, 10);
+
+				mdecrypt_generic(td2, ciphertext, 10);
+
+#ifdef DEBUG
+			printf("\nplaintext:  ");
+			for (j = 0; j < 10; j++) {
+				printf("%.2x", ciphertext[j]);
+			}
+			printf("\n");
+#endif
+
+				if (memcmp(ciphertext, plaintext, 10) != 0) {
+					printf("failed internally\n");
+					x = 1;
+					mcrypt_generic_end(td);
+					mcrypt_generic_end(td2);
+					free(name);
+					free(keyword);
+					free(ciphertext);
+					free(plaintext);
+					continue;
+				}
+				printf("ok\n");
+
+				mcrypt_generic_end(td);
+				mcrypt_generic_end(td2);
+				free(name);
+				free(keyword);
+				free(ciphertext);
+				free(plaintext);
+
+				continue;
+			}
 			IV = malloc(mcrypt_get_block_size(i));
 			memset(IV, '\234', mcrypt_get_block_size(i));
 			memset(IV, '\123', mcrypt_get_block_size(i) / 2);
@@ -46,12 +127,12 @@ int main()
 			if (td < 0)
 				continue;
 
-			name = get_algorithms_name(i);
+			name = mcrypt_get_algorithms_name(i);
 			if (name == NULL)
 				continue;
 			printf("Algorithm: %s... ", name);
 
-			name2 = get_modes_name(mode);
+			name2 = mcrypt_get_modes_name(mode);
 			if (name2 == NULL)
 				continue;
 			printf("Mode: %s... ", name2);
@@ -64,16 +145,16 @@ int main()
 
 #ifdef DEBUG
 			printf("plaintext:  ");
-			*/for (j = 0; j < mcrypt_get_block_size(i); j++) {
+			for (j = 0; j < mcrypt_get_block_size(i); j++) {
 				printf("%.2x", ciphertext[j]);
-			*/}
+			}
 #endif
 
 #ifdef DEBUG
 			printf("\nkeyword:    ");
-			*/for (j = 0; j < mcrypt_get_key_size(i); j++) {
+			for (j = 0; j < mcrypt_get_key_size(i); j++) {
 				printf("%.2x", keyword[j]);
-			*/}
+			}
 #endif
 
 			mcrypt_generic(td, ciphertext,
